@@ -45,6 +45,7 @@
 #include <fcntl.h>
 #include <linux/if_tun.h>
 #include <ctype.h>
+#include <ifaddrs.h>
 
 #include "connman.h"
 
@@ -234,7 +235,7 @@ char *connman_inet_ifname(int index)
 	if (err < 0)
 		return NULL;
 
-	return strdup(ifr.ifr_name);
+	return g_strdup(ifr.ifr_name);
 }
 
 short int connman_inet_ifflags(int index)
@@ -357,95 +358,6 @@ done:
 	return err;
 }
 
-static char *index2addr(int index)
-{
-	struct ifreq ifr;
-	struct ether_addr eth;
-	char *str;
-	int sk, err;
-
-	if (index < 0)
-		return NULL;
-
-	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return NULL;
-
-	memset(&ifr, 0, sizeof(ifr));
-	ifr.ifr_ifindex = index;
-
-	err = ioctl(sk, SIOCGIFNAME, &ifr);
-
-	if (err == 0)
-		err = ioctl(sk, SIOCGIFHWADDR, &ifr);
-
-	close(sk);
-
-	if (err < 0)
-		return NULL;
-
-	str = malloc(18);
-	if (!str)
-		return NULL;
-
-	memcpy(&eth, &ifr.ifr_hwaddr.sa_data, sizeof(eth));
-	snprintf(str, 18, "%02X:%02X:%02X:%02X:%02X:%02X",
-						eth.ether_addr_octet[0],
-						eth.ether_addr_octet[1],
-						eth.ether_addr_octet[2],
-						eth.ether_addr_octet[3],
-						eth.ether_addr_octet[4],
-						eth.ether_addr_octet[5]);
-
-	return str;
-}
-
-static char *index2ident(int index, const char *prefix)
-{
-	struct ifreq ifr;
-	struct ether_addr eth;
-	char *str;
-	int sk, err, len;
-
-	if (index < 0)
-		return NULL;
-
-	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return NULL;
-
-	memset(&ifr, 0, sizeof(ifr));
-	ifr.ifr_ifindex = index;
-
-	err = ioctl(sk, SIOCGIFNAME, &ifr);
-
-	if (err == 0)
-		err = ioctl(sk, SIOCGIFHWADDR, &ifr);
-
-	close(sk);
-
-	if (err < 0)
-		return NULL;
-
-	len = prefix ? strlen(prefix) + 18 : 18;
-
-	str = malloc(len);
-	if (!str)
-		return NULL;
-
-	memcpy(&eth, &ifr.ifr_hwaddr.sa_data, sizeof(eth));
-	snprintf(str, len, "%s%02x%02x%02x%02x%02x%02x",
-						prefix ? prefix : "",
-						eth.ether_addr_octet[0],
-						eth.ether_addr_octet[1],
-						eth.ether_addr_octet[2],
-						eth.ether_addr_octet[3],
-						eth.ether_addr_octet[4],
-						eth.ether_addr_octet[5]);
-
-	return str;
-}
-
 connman_bool_t connman_inet_is_cfg80211(int index)
 {
 	connman_bool_t result = FALSE;
@@ -476,90 +388,6 @@ done:
 	return result;
 }
 
-struct connman_device *connman_inet_create_device(int index)
-{
-	enum connman_device_type type;
-	struct connman_device *device;
-	char *devname, *ident = NULL;
-	char *addr = NULL, *name = NULL;
-
-	if (index < 0)
-		return NULL;
-
-	devname = connman_inet_ifname(index);
-	if (devname == NULL)
-		return NULL;
-
-	if (__connman_device_isfiltered(devname) == TRUE) {
-		connman_info("Ignoring interface %s (filtered)", devname);
-		free(devname);
-		return NULL;
-	}
-
-	type = __connman_rtnl_get_device_type(index);
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-		connman_info("Ignoring interface %s (type unknown)", devname);
-		free(devname);
-		return NULL;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-	case CONNMAN_DEVICE_TYPE_GADGET:
-	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-		name = index2ident(index, "");
-		addr = index2addr(index);
-		break;
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_CELLULAR:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-		name = strdup(devname);
-		break;
-	}
-
-	device = connman_device_create(name, type);
-	if (device == NULL)
-		goto done;
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-	case CONNMAN_DEVICE_TYPE_GPS:
-		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-	case CONNMAN_DEVICE_TYPE_GADGET:
-		ident = index2ident(index, NULL);
-		break;
-	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-		ident = index2ident(index, NULL);
-		break;
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-		break;
-	case CONNMAN_DEVICE_TYPE_CELLULAR:
-		ident = index2ident(index, NULL);
-		break;
-	}
-
-	connman_device_set_index(device, index);
-	connman_device_set_interface(device, devname);
-
-	if (ident != NULL) {
-		connman_device_set_ident(device, ident);
-		free(ident);
-	}
-
-	connman_device_set_string(device, "Address", addr);
-
-done:
-	free(devname);
-	free(name);
-	free(addr);
-
-	return device;
-}
-
 struct in6_ifreq {
 	struct in6_addr ifr6_addr;
 	__u32 ifr6_prefixlen;
@@ -677,21 +505,24 @@ int connman_inet_add_network_route(int index, const char *host,
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d host %s gateway %s netmask %s", index,
 		host, gateway, netmask);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -727,13 +558,16 @@ int connman_inet_add_network_route(int index, const char *host,
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCADDRT, &rt);
-	if (err < 0)
-		connman_error("Adding host route failed (%s)",
-							strerror(errno));
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
 
 	close(sk);
 
+out:
+	if (err < 0)
+		connman_error("Adding host route failed (%s)",
+							strerror(-err));
+
 	return err;
 }
 
@@ -742,20 +576,23 @@ int connman_inet_del_network_route(int index, const char *host)
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d host %s", index, host);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -770,13 +607,16 @@ int connman_inet_del_network_route(int index, const char *host)
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCDELRT, &rt);
-	if (err < 0)
-		connman_error("Deleting host route failed (%s)",
-							strerror(errno));
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
 
 	close(sk);
 
+out:
+	if (err < 0)
+		connman_error("Deleting host route failed (%s)",
+							strerror(-err));
+
 	return err;
 }
 
@@ -784,7 +624,7 @@ int connman_inet_del_ipv6_network_route(int index, const char *host,
 						unsigned char prefix_len)
 {
 	struct in6_rtmsg rt;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d host %s", index, host);
 
@@ -795,9 +635,10 @@ int connman_inet_del_ipv6_network_route(int index, const char *host,
 
 	rt.rtmsg_dst_len = prefix_len;
 
-	err = inet_pton(AF_INET6, host, &rt.rtmsg_dst);
-	if (err < 0)
+	if (inet_pton(AF_INET6, host, &rt.rtmsg_dst) < 0) {
+		err = -errno;
 		goto out;
+	}
 
 	rt.rtmsg_flags = RTF_UP | RTF_HOST;
 
@@ -806,16 +647,19 @@ int connman_inet_del_ipv6_network_route(int index, const char *host,
 
 	sk = socket(AF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (sk < 0) {
-		err = -1;
+		err = -errno;
 		goto out;
 	}
 
-	err = ioctl(sk, SIOCDELRT, &rt);
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
+
 	close(sk);
+
 out:
 	if (err < 0)
 		connman_error("Del IPv6 host route error (%s)",
-						strerror(errno));
+						strerror(-err));
 
 	return err;
 }
@@ -830,7 +674,7 @@ int connman_inet_add_ipv6_network_route(int index, const char *host,
 					unsigned char prefix_len)
 {
 	struct in6_rtmsg rt;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d host %s gateway %s", index, host, gateway);
 
@@ -841,9 +685,10 @@ int connman_inet_add_ipv6_network_route(int index, const char *host,
 
 	rt.rtmsg_dst_len = prefix_len;
 
-	err = inet_pton(AF_INET6, host, &rt.rtmsg_dst);
-	if (err < 0)
+	if (inet_pton(AF_INET6, host, &rt.rtmsg_dst) < 0) {
+		err = -errno;
 		goto out;
+	}
 
 	rt.rtmsg_flags = RTF_UP | RTF_HOST;
 
@@ -857,16 +702,19 @@ int connman_inet_add_ipv6_network_route(int index, const char *host,
 
 	sk = socket(AF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (sk < 0) {
-		err = -1;
+		err = -errno;
 		goto out;
 	}
 
-	err = ioctl(sk, SIOCADDRT, &rt);
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
+
 	close(sk);
+
 out:
 	if (err < 0)
 		connman_error("Set IPv6 host route error (%s)",
-						strerror(errno));
+						strerror(-err));
 
 	return err;
 }
@@ -880,7 +728,7 @@ int connman_inet_add_ipv6_host_route(int index, const char *host,
 int connman_inet_set_ipv6_gateway_address(int index, const char *gateway)
 {
 	struct in6_rtmsg rt;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d gateway %s", index, gateway);
 
@@ -889,9 +737,10 @@ int connman_inet_set_ipv6_gateway_address(int index, const char *gateway)
 
 	memset(&rt, 0, sizeof(rt));
 
-	err = inet_pton(AF_INET6, gateway, &rt.rtmsg_gateway);
-	if (err < 0)
+	if (inet_pton(AF_INET6, gateway, &rt.rtmsg_gateway) < 0) {
+		err = -errno;
 		goto out;
+	}
 
 	rt.rtmsg_flags = RTF_UP | RTF_GATEWAY;
 	rt.rtmsg_metric = 1;
@@ -900,16 +749,19 @@ int connman_inet_set_ipv6_gateway_address(int index, const char *gateway)
 
 	sk = socket(AF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (sk < 0) {
-		err = -1;
+		err = -errno;
 		goto out;
 	}
 
-	err = ioctl(sk, SIOCADDRT, &rt);
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
+
 	close(sk);
+
 out:
 	if (err < 0)
 		connman_error("Set default IPv6 gateway error (%s)",
-						strerror(errno));
+						strerror(-err));
 
 	return err;
 }
@@ -917,7 +769,7 @@ out:
 int connman_inet_clear_ipv6_gateway_address(int index, const char *gateway)
 {
 	struct in6_rtmsg rt;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d gateway %s", index, gateway);
 
@@ -926,9 +778,10 @@ int connman_inet_clear_ipv6_gateway_address(int index, const char *gateway)
 
 	memset(&rt, 0, sizeof(rt));
 
-	err = inet_pton(AF_INET6, gateway, &rt.rtmsg_gateway);
-	if (err < 0)
+	if (inet_pton(AF_INET6, gateway, &rt.rtmsg_gateway) < 0) {
+		err = -errno;
 		goto out;
+	}
 
 	rt.rtmsg_flags = RTF_UP | RTF_GATEWAY;
 	rt.rtmsg_metric = 1;
@@ -937,16 +790,19 @@ int connman_inet_clear_ipv6_gateway_address(int index, const char *gateway)
 
 	sk = socket(AF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (sk < 0) {
-		err = -1;
+		err = -errno;
 		goto out;
 	}
 
-	err = ioctl(sk, SIOCDELRT, &rt);
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
+
 	close(sk);
+
 out:
 	if (err < 0)
 		connman_error("Clear default IPv6 gateway error (%s)",
-						strerror(errno));
+						strerror(-err));
 
 	return err;
 }
@@ -956,20 +812,23 @@ int connman_inet_set_gateway_address(int index, const char *gateway)
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d gateway %s", index, gateway);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -992,13 +851,16 @@ int connman_inet_set_gateway_address(int index, const char *gateway)
 	addr.sin_addr.s_addr = INADDR_ANY;
 	memcpy(&rt.rt_genmask, &addr, sizeof(rt.rt_genmask));
 
-	err = ioctl(sk, SIOCADDRT, &rt);
-	if (err < 0)
-		connman_error("Setting default gateway route failed (%s)",
-							strerror(errno));
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
 
 	close(sk);
 
+out:
+	if (err < 0)
+		connman_error("Setting default gateway route failed (%s)",
+							strerror(-err));
+
 	return err;
 }
 
@@ -1007,20 +869,23 @@ int connman_inet_set_gateway_interface(int index)
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d", index);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -1038,11 +903,15 @@ int connman_inet_set_gateway_interface(int index)
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCADDRT, &rt);
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
+
+	close(sk);
+
+out:
 	if (err < 0)
 		connman_error("Setting default interface route failed (%s)",
-							strerror(errno));
-	close(sk);
+							strerror(-err));
 
 	return err;
 }
@@ -1053,20 +922,23 @@ int connman_inet_set_ipv6_gateway_interface(int index)
 	struct rtentry rt;
 	struct sockaddr_in6 addr;
 	const struct in6_addr any = IN6ADDR_ANY_INIT;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d", index);
 
 	sk = socket(PF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -1084,11 +956,15 @@ int connman_inet_set_ipv6_gateway_interface(int index)
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCADDRT, &rt);
+	if (ioctl(sk, SIOCADDRT, &rt) < 0 && errno != EEXIST)
+		err = -errno;
+
+	close(sk);
+
+out:
 	if (err < 0)
 		connman_error("Setting default interface route failed (%s)",
-							strerror(errno));
-	close(sk);
+							strerror(-err));
 
 	return err;
 }
@@ -1098,20 +974,23 @@ int connman_inet_clear_gateway_address(int index, const char *gateway)
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d gateway %s", index, gateway);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -1134,13 +1013,16 @@ int connman_inet_clear_gateway_address(int index, const char *gateway)
 	addr.sin_addr.s_addr = INADDR_ANY;
 	memcpy(&rt.rt_genmask, &addr, sizeof(rt.rt_genmask));
 
-	err = ioctl(sk, SIOCDELRT, &rt);
-	if (err < 0)
-		connman_error("Removing default gateway route failed (%s)",
-							strerror(errno));
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
 
 	close(sk);
 
+out:
+	if (err < 0)
+		connman_error("Removing default gateway route failed (%s)",
+							strerror(-err));
+
 	return err;
 }
 
@@ -1149,20 +1031,23 @@ int connman_inet_clear_gateway_interface(int index)
 	struct ifreq ifr;
 	struct rtentry rt;
 	struct sockaddr_in addr;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d", index);
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -1180,11 +1065,15 @@ int connman_inet_clear_gateway_interface(int index)
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCDELRT, &rt);
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
+
+	close(sk);
+
+out:
 	if (err < 0)
 		connman_error("Removing default interface route failed (%s)",
-							strerror(errno));
-	close(sk);
+							strerror(-err));
 
 	return err;
 }
@@ -1195,20 +1084,23 @@ int connman_inet_clear_ipv6_gateway_interface(int index)
 	struct rtentry rt;
 	struct sockaddr_in6 addr;
 	const struct in6_addr any = IN6ADDR_ANY_INIT;
-	int sk, err;
+	int sk, err = 0;
 
 	DBG("index %d", index);
 
 	sk = socket(PF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return -1;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_ifindex = index;
 
 	if (ioctl(sk, SIOCGIFNAME, &ifr) < 0) {
+		err = -errno;
 		close(sk);
-		return -1;
+		goto out;
 	}
 
 	DBG("ifname %s", ifr.ifr_name);
@@ -1226,11 +1118,15 @@ int connman_inet_clear_ipv6_gateway_interface(int index)
 
 	rt.rt_dev = ifr.ifr_name;
 
-	err = ioctl(sk, SIOCDELRT, &rt);
+	if (ioctl(sk, SIOCDELRT, &rt) < 0 && errno != ESRCH)
+		err = -errno;
+
+	close(sk);
+
+out:
 	if (err < 0)
 		connman_error("Removing default interface route failed (%s)",
-							strerror(errno));
-	close(sk);
+							strerror(-err));
 
 	return err;
 }
@@ -1288,59 +1184,63 @@ connman_bool_t connman_inet_compare_subnet(int index, const char *host)
 int connman_inet_remove_from_bridge(int index, const char *bridge)
 {
 	struct ifreq ifr;
-	int sk, err;
+	int sk, err = 0;
 
 	if (bridge == NULL)
 		return -EINVAL;
 
 	sk = socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return sk;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, bridge, IFNAMSIZ - 1);
 	ifr.ifr_ifindex = index;
 
-	err = ioctl(sk, SIOCBRDELIF, &ifr);
+	if (ioctl(sk, SIOCBRDELIF, &ifr) < 0)
+		err = -errno;
 
 	close(sk);
 
-	if (err < 0) {
+out:
+	if (err < 0)
 		connman_error("Remove interface from bridge error %s",
-							strerror(errno));
-		return err;
-	}
+							strerror(-err));
 
-	return 0;
+	return err;
 }
 
 int connman_inet_add_to_bridge(int index, const char *bridge)
 {
 	struct ifreq ifr;
-	int sk, err;
+	int sk, err = 0;
 
 	if (bridge == NULL)
 		return -EINVAL;
 
 	sk = socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, 0);
-	if (sk < 0)
-		return sk;
+	if (sk < 0) {
+		err = -errno;
+		goto out;
+	}
 
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, bridge, IFNAMSIZ - 1);
 	ifr.ifr_ifindex = index;
 
-	err = ioctl(sk, SIOCBRADDIF, &ifr);
+	if (ioctl(sk, SIOCBRADDIF, &ifr) < 0)
+		err = -errno;
 
 	close(sk);
 
-	if (err < 0) {
+out:
+	if (err < 0)
 		connman_error("Add interface to bridge error %s",
-							strerror(errno));
-		return err;
-	}
+							strerror(-err));
 
-	return 0;
+	return err;
 }
 
 int connman_inet_set_mtu(int index, int mtu)
@@ -1382,9 +1282,7 @@ int connman_inet_setup_tunnel(char *tunnel, int mtu)
 	index = if_nametoindex(tunnel);
 
 	err = connman_inet_set_mtu(index, mtu);
-	if (err < 0)
-		return err;
-	else if (err)
+	if (err != 0)
 		goto done;
 
 	memset(&ifr, 0, sizeof(ifr));
@@ -2071,13 +1969,7 @@ static int inet_rtnl_recv(GIOChannel *chan, gpointer user_data)
 
 		rtnl_data->callback(h, rtnl_data->user_data);
 
-		if (rtnl_data->rtnl_timeout > 0) {
-			g_source_remove(rtnl_data->rtnl_timeout);
-			rtnl_data->rtnl_timeout = 0;
-		}
-
-		__connman_inet_rtnl_close(rth);
-		g_free(rth);
+		inet_rtnl_cleanup(rtnl_data);
 	}
 
 	return 0;
@@ -2461,7 +2353,7 @@ char **__connman_inet_get_running_interfaces(void)
 		result[count++] = g_strdup(r->ifr_name);
 	}
 
-	free(ifr);
+	g_free(ifr);
 
 	if (count < numif)
 		result = g_try_realloc(result, (count + 1) * sizeof(char *));
@@ -2470,6 +2362,72 @@ char **__connman_inet_get_running_interfaces(void)
 
 error:
 	close(sk);
-	free(ifr);
+	g_free(ifr);
 	return NULL;
 }
+
+connman_bool_t connman_inet_is_ipv6_supported()
+{
+	int sk;
+
+	sk = socket(PF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (sk < 0)
+		return FALSE;
+
+	close(sk);
+	return TRUE;
+}
+
+int __connman_inet_get_interface_address(int index, int family, void *address)
+{
+	struct ifaddrs *ifaddr, *ifa;
+	int err = -ENOENT;
+	char name[IF_NAMESIZE];
+
+	if (if_indextoname(index, name) == NULL)
+		return -EINVAL;
+
+	DBG("index %d interface %s", index, name);
+
+	if (getifaddrs(&ifaddr) < 0) {
+		err = -errno;
+		DBG("Cannot get addresses err %d/%s", err, strerror(-err));
+		return err;
+	}
+
+	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+		if (ifa->ifa_addr == NULL)
+			continue;
+
+		if (strncmp(ifa->ifa_name, name, IF_NAMESIZE) == 0 &&
+					ifa->ifa_addr->sa_family == family) {
+			if (family == AF_INET) {
+				struct sockaddr_in *in4 = (struct sockaddr_in *)
+					ifa->ifa_addr;
+				if (in4->sin_addr.s_addr == INADDR_ANY)
+					continue;
+				memcpy(address, &in4->sin_addr,
+							sizeof(struct in_addr));
+			} else if (family == AF_INET6) {
+				struct sockaddr_in6 *in6 =
+					(struct sockaddr_in6 *)ifa->ifa_addr;
+				if (memcmp(&in6->sin6_addr, &in6addr_any,
+						sizeof(struct in6_addr)) == 0)
+					continue;
+				memcpy(address, &in6->sin6_addr,
+						sizeof(struct in6_addr));
+
+			} else {
+				err = -EINVAL;
+				goto out;
+			}
+
+			err = 0;
+			break;
+		}
+	}
+
+out:
+	freeifaddrs(ifaddr);
+	return err;
+}
